<?php
defined('BASEPATH') or exit('No direct script access allowed');

/*
Module Name: Spark Filters
Description: Spark Filters Module - Filter Customers and Leads 
Author: Spark Apps
Version: 1.0.1
Requires at least: 1.0.*
*/

define('Spark_FILTERS_MODULE_NAME', 'spark_filters');

$CI = &get_instance();

hooks()->add_action('admin_init', 'spark_filters_init_menu_items');
hooks()->add_action('admin_init', 'spark_filters_permissions');

/**
* Load the module helper
*/
$CI->load->helper(Spark_FILTERS_MODULE_NAME . '/spark_filters');

/**
* Load the module Model
*/
$CI->load->model(Spark_FILTERS_MODULE_NAME . '/spark_filters_model');

/**
* Register activation module hook
*/
register_activation_hook(Spark_FILTERS_MODULE_NAME, 'spark_filters_activation_hook');

function spark_filters_activation_hook()
{
	$CI = &get_instance();
	require_once(__DIR__ . '/install.php');
}

/**
* Register language files, must be registered if the module is using languages
*/
register_language_files(Spark_FILTERS_MODULE_NAME, [Spark_FILTERS_MODULE_NAME]);

/**
 * Init menu setup module menu items in setup in admin_init hook
 * @return null
 */
function spark_filters_init_menu_items()
{
	/**
	* If the logged in user is administrator, add custom Reports in Sidebar, if want to add menu in Setup then Write Setup instead of sidebar in menu ceation
	*/
	if (is_admin() || has_permission('spark_filters', '', 'view')) {
		$CI = &get_instance();
		$CI->app_menu->add_sidebar_menu_item('lead-filters', [
			'collapse'	=> true,
			'icon'		=> 'fa fa-filter',
			'name'		=> _l('spark_filters_menu'),
			'position'	=> 35,
		]);
		$CI->app_menu->add_sidebar_children_item('lead-filters', [
			'slug'		=> 'spark-lead-filter-options',
			'name'		=> _l('spark_submenu_lead_filters'),
			'href'		=> admin_url('spark_filters/leads_filter'),
			'position'	=> 5,
		]);
		$CI->app_menu->add_sidebar_children_item('lead-filters', [
			'slug'		=> 'spark-client-filter-options',
			'name'		=> _l('submenu_customers_filters'),
			'href'		=> admin_url('spark_filters/customers_filter'),
			'position'	=> 10,
		]);
	}
}
function spark_filters_permissions()
{
	$capabilities = [];
	$capabilities['capabilities'] = [
		'view'   => _l('permission_view') . '(' . _l('permission_global') . ')',
		'create' => _l('permission_create'),
	];
	register_staff_capabilities('spark_filters', $capabilities, _l('spark_filters'));
}
